/*
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the
 * copyright holders. 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.glite.apel.pbs;

import java.io.FileNotFoundException;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.glite.apel.core.ApelException;
import org.glite.apel.core.Log;
import org.glite.apel.core.config.map.CLMap;
import org.glite.apel.core.config.map.FileMap;
import org.glite.apel.core.config.parser.CLParser;
import org.glite.apel.core.config.parser.Config;
import org.glite.apel.core.config.parser.ConfigurationException;
import org.glite.apel.core.config.parser.XMLParser;
import org.glite.apel.core.log.parser.GkLogParser;
import org.glite.apel.core.processor.DeleteProcessor;
import org.glite.apel.core.processor.DBProcessor;
import org.glite.apel.core.processor.GkLogProcessor;
import org.glite.apel.core.processor.SpecProcessor;
import org.glite.apel.core.processor.BlahdLogProcessor;

/**
 * The main entry point for starting the apel accounting application.
 */
public class ApelLogParser {
	
    /**
     * Using log4j logging.
     */
    private static final Logger LOG = 
    	Logger.getLogger(ApelLogParser.class.getName());
    
    /**
     * The script name used.
     */
    private static final String PROG_NAME = "apel-pbs-log-parser";
    
    /** 
     * Single reference that points to all the properties/configs
     * for this application. 
     */
    private Config m_config;
    
    /**
     * Sets-up the user configs for the apel application.
     */
    public ApelLogParser(String[] args) {
        Log.setProgramName(PROG_NAME);
    	setUp(args);
    }
    
    /**
     * Kick starts the apel application.
     */
    private void run() {
        String url = m_config.getProperty(FileMap.PROP_DB_URL);
        String user = m_config.getProperty(FileMap.PROP_DB_USERNAME);
        String password = m_config.getProperty(FileMap.PROP_DB_PASSWORD);
        String siteName = m_config.getProperty(FileMap.PROP_SITE);
        try {
            Log.info(LOG, "------ Starting the apel application ------");
            if (m_config.isOptionSet(FileMap.OPT_DB_DELETE_PROCESSOR)) {
                DeleteProcessor deleteProc = new DeleteProcessor(url, user, password, siteName);
                boolean deleteAll = m_config.isOptionSet(FileMap.OPT_CLEAN_ALL_FROM_TABLES);
                deleteProc.run(deleteAll);
            }            
            if (m_config.isOptionSet(FileMap.OPT_DB_PROCESSOR)) {
                DBProcessor dbProc = new DBProcessor(url, user, password, siteName);
                boolean inspectTables = m_config.isOptionSet(FileMap.OPT_INSPECT_TABLES);
                dbProc.run(inspectTables);
            }
            if (m_config.isOptionSet(FileMap.OPT_SPEC_PROCESSOR)) {
            	runSpecProcessor(url, user, password, siteName);
            }
            if (m_config.isOptionSet(FileMap.OPT_EVENT_LOG_PROCESSOR)) {
                runEventProcessor(url, user, password, siteName);
            }
            if (m_config.isOptionSet(FileMap.OPT_GK_LOG_PROCESSOR)) {
            	runGkProcessor(url, user, password, siteName);
            }
            if (m_config.isOptionSet(FileMap.OPT_BLAHD_LOG_PROCESSOR)) {
                runBlahdProcessor(url, user, password, siteName);
            }
            Log.info(LOG, "------ Processing finished ------");
            System.exit(0);
        } catch (ApelException ae) {
            Log.die(LOG, ae);
        } catch (Exception e) {
            Log.die(LOG, "Internal program error - please report bug", e);
        }
    }
    
    /**
     * Runs the spec processor (to setup a spec int/float value).
     */
        private void runSpecProcessor(String url, String user, String password, 
         String siteName) throws ApelException {
             SpecProcessor spec = new SpecProcessor(url, user, password, siteName);
             String defaultSpec = m_config.getOptionalProperty(FileMap.PROP_USE_DEFAULT_SPEC);
             String giisHost = m_config.getOptionalProperty(FileMap.PROP_GIIS);
             if (defaultSpec != null) {
              spec.setDefaultSpec(defaultSpec);
             } else if (giisHost != null) {
              spec.setLDAPSpec(giisHost);
             } else {
               throw new ApelException("Neither a GIIS host nor a default CPU " +
                "SPEC value was specified, please check config file");
             }
        }

    /**
     * Runs the event processor (to parse all event logs).
     */
       private void runEventProcessor(String url, String user, String password, 
        String siteName) throws ApelException {
         String eventLogDir = m_config.getOptionalProperty(FileMap.PROP_EVENT_LOG_DIR);
         boolean searchSubDirs = m_config.isOptionSet(FileMap.OPT_EVENT_SEARCH_ALL);
         boolean reprocessLogs = m_config.isOptionSet(FileMap.OPT_EVENT_LOG_REPROCESS);
         Vector extraLogs = m_config.getPropertyValues(FileMap.PROP_EXTRA_EVENT_LOGS);
         String timezone = m_config.getOptionalProperty(FileMap.PROP_TIME_ZONE);
	 String submitHost = m_config.getProperty(FileMap.PROP_SUBMIT_HOST);
         EventLogProcessor eventProc = new EventLogProcessor(url, user, password, 
          siteName, eventLogDir, searchSubDirs, reprocessLogs, extraLogs, timezone, submitHost);
         eventProc.processLogs();
         eventProc.close();
       }

    /**
     * Runs the gk processor (to parse all gk logs).
     */
       private void runGkProcessor(String url, String user, String password, 
        String siteName) throws ApelException {
        String gkLogDir  = m_config.getOptionalProperty(FileMap.PROP_GKLOG);
        String msgLogDir = m_config.getOptionalProperty(FileMap.PROP_MSLOG);
        boolean searchSubDirs = m_config.isOptionSet(FileMap.OPT_GK_SEARCH_ALL);
        boolean reprocessLogs = m_config.isOptionSet(FileMap.OPT_GK_LOG_REPROCESS);
        Vector extraGkLogs = m_config.getPropertyValues(FileMap.PROP_EXTRA_GK_LOGS);
        Vector extraMsgLogs = m_config.getPropertyValues(FileMap.PROP_EXTRA_MSG_LOG_FILES);
        GkLogProcessor gkProc = new GkLogProcessor(url, user, password, siteName, gkLogDir, 
         msgLogDir, searchSubDirs, reprocessLogs, extraGkLogs, extraMsgLogs, GkLogParser.PBS_FORMAT);
        gkProc.processLogs();
        gkProc.close();
       }

    /**
     * Runs the Blahd Log processor (to parse all Blahd logs).
     */
       private void runBlahdProcessor(String url, String user, String password,
        String siteName) throws ApelException {
          String eventLogDir    = m_config.getOptionalProperty(FileMap.PROP_BLAHDLOG);
          String eventLogPrefix = m_config.getOptionalProperty(FileMap.PROP_BLAHDLOG_PREFIX);
          String submitHost = m_config.getProperty(FileMap.PROP_SUBMIT_HOST);
          boolean searchSubDirs = m_config.isOptionSet(FileMap.OPT_BLAHD_SEARCH_ALL);
          boolean reprocessLogs = m_config.isOptionSet(FileMap.OPT_BLAHD_LOG_REPROCESS);
          Vector extraLogs = m_config.getPropertyValues(FileMap.PROP_EXTRA_BLAHD_LOGS);
          BlahdLogProcessor blahdProc = new BlahdLogProcessor(url, user, password, 
           siteName, eventLogDir, searchSubDirs, reprocessLogs, extraLogs, eventLogPrefix, submitHost);
          blahdProc.processLogs();
          blahdProc.close();
        }

   /**
     * Intialises input parameter defined on the command line 
     * and then attempts to read-in configs from a property file.
     * The location of the property file is defined on the command line
     */
    private void setUp(String[] args) {
        String configFileName = null;
        try {
            if (args.length == 0) {
                help();
            }
            CLParser clParser = new CLParser(args); 
            Config clConfig = clParser.parse(new CLMap());
            processStandardOptions(clConfig);
            configFileName = 
                clConfig.getOptionalProperty(CLMap.PROP_CONFIG_FILE);
            if (configFileName == null) {
                Log.die(LOG, "No config file defined, use -h or " +
                    "--help for more info");
            }
            LOG.debug("using config file: " + configFileName);
            XMLParser xmlParser = new XMLParser(configFileName);
            m_config = xmlParser.parse(new FileMap());
        } catch (ConfigurationException e) {
            Log.die(LOG, e.getMessage());
        } catch (FileNotFoundException e) {
            Log.die(LOG, "File not found: " + configFileName);
        }
        if (m_config.isOptionSet(FileMap.OPT_QUIET)) {
            Log.setQuiet(false);
        }
        Vector hideProperties = new Vector();
        hideProperties.add(FileMap.PROP_DB_PASSWORD);
        Log.info(LOG, "Read-in configuration: " + 
            m_config.toString(hideProperties));
    }
    
    /**
     * Checks and processes 'help', 'version' and 'quiet' options.
     */
    private void processStandardOptions(Config config) {
        if (config.isOptionSet(CLMap.OPT_HELP)) {
            help();
        }
        if (config.isOptionSet(CLMap.OPT_VERSION)) {
            showVersion();
        }
    }

    /**
     * Displays help information and terminate.
     */
    private void help() {
        System.out.println();
        System.out.print("The " + PROG_NAME + " is an accounting tool that ");
        System.out.print("parses PBS, Globus Gate Keeper (LCG), Blahd accounting log (gLite)");
        System.out.print("and Job manager log files. ");
        System.out.print("Data extracted from the log files is ");
        System.out.print("then stored in a user defined database. ");
        System.out.print("Please refer to the user guide ");
        System.out.println("for full installation and setup instructions. ");
        System.out.println();
        System.out.println("Usage: " + PROG_NAME + " [option]...");
        System.out.println();
        System.out.println("\t-f, --config[=filename]");
        System.out.println();
        System.out.println("\tthe config file to be used");
        System.out.println();
        System.out.println("\t-h, --help"); 
        System.out.println();
        System.out.println("\tdisplay this help and exit");
        System.out.println();
        System.out.println("\t-v, --version"); 
        System.out.println();
        System.out.println("\toutput version information and exit");
        System.out.println();
        System.out.println("Examples:");
        System.out.println();
        System.out.println("\t" + PROG_NAME + " -f /home/rutabaga/config.xml");
        System.out.println();
        System.out.println("Report bugs to Savannah at " +
            "https://savannah.cern.ch/bugs/?group=jra1mdw");
        System.out.println();
        System.exit(0);
    }
    
    /**
     * Display program version number and exits.
     */
    private void showVersion() {
        // CVS will replace this string with the correct information
        String cvsTag = "$Name: glite-apel-pbs_R_2_0_6_5 $";
        if (cvsTag.matches(".Name:  .")) {
            cvsTag = "Development";
        } else {
            cvsTag = cvsTag.replaceFirst(".Name: ", "");
            if (cvsTag.length() >= 2) {
                cvsTag = cvsTag.substring(0, cvsTag.length() - 2);
            }
        }
        System.out.println();
        System.out.println(PROG_NAME + " version " + cvsTag);
        System.out.println("Copyright (c) Members of the EGEE Collaboration. " +
            "2004. See http://eu-egee.org/partners/ for details on the " +
            "copyright holders." +
            "For license conditions see For license conditions see the " +
            "license file or http://eu-egee.org/license.html");
        System.out.println();
        System.exit(0);
    }    
    
    /**
     * Create an Apel object and call its start method. This is the program
     * entry point for processing of data. The archiver program has a
     * different entry point.
     */
    public static void main(String[] args) {
    	ApelLogParser apel = new ApelLogParser(args);
    	apel.run();
    }
    
}

