/*
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the
 * copyright holders. 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.glite.apel.pbs;

import org.apache.log4j.Logger;
import org.glite.apel.core.Log;
import org.glite.apel.core.Time;
import org.glite.apel.core.TimeInterval;
import org.glite.apel.core.log.type.EventRecord;
import org.glite.apel.core.log.type.EventRecordIterator;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.Vector;
import java.util.regex.Pattern;
import java.util.zip.GZIPInputStream;

/**
 * A file containing a PBS event log
 */
class EventLogParser extends File {
    
    /**
     * Using log4j logging tool.
     */
    protected static final Logger LOG = 
        Logger.getLogger(EventLogParser.class.getName());
    
    /**
     * Buffer which reads-in data from the log file.
     */
    protected BufferedReader m_reader;
    
    /**
     * Holds all event records read-in from the event log file.
     */
    protected Vector m_eventRecords;
    
    /** 
     * The format used to encode the current date. 
     */
    private static final String EVENT_DATE_FORMAT = "yyyyMMdd";
    
    /** 
     * UTC constant used when converting the current date. 
     */
    private static final String UTC = "UTC";
    
    /**
     * Formatter converts the current date into the same
     * format used by the event log file.
     */
    private SimpleDateFormat m_dateFormatter;
    
    /**
     * The timezone to be used when interpreting time stamps 
     * parsed from the log files.
     */
    private String m_timezone;

     /**
     * The file object used to interact with the log file itself
     */
    private File m_file;
    
    /**
     * Pattern used to chop up a line of data that matches an
     * EventRecord into a convenient list.
     */
    private Pattern m_delimiter;
    
    /**
     * Pattern used to match a pbs event record in the log file.
     */
    private Pattern m_endEventPattern;
   
    /**
     * The value for submitHost taken from the configuration file
     */
    private String m_submitHost;
 
    /**
     * Creates a new event log file.
     */
    ////////////////////////////////////////////
    // timezone is taken from EventLogProcessor <Timezone>
    // If it is not explicitly set, timezone will be null
    // see section 4 of manual for explanation
    ////////////////////////////////////////////
    EventLogParser(File file, String timezone, String submitHost) {
        super(file.getPath());
        m_eventRecords = new Vector();
        m_submitHost = submitHost;
        m_file = file;

        /////////////////////////////////
        // Logic moved to isActiveFile //
        /////////////////////////////////
        //m_timezone = timezone; 
        //m_dateFormatter = new SimpleDateFormat(EVENT_DATE_FORMAT);
        //Basically m_dateFormatter will convert datetimes to UTC.
        //This is wrong for isActiveFile which looks at the YYYYMMDD filename of the log.
        //m_dateFormatter.setTimeZone(TimeZone.getTimeZone(UTC));
        /////////////////////////////////
        m_delimiter = Pattern.compile("[ ;]");
        String prefixString = "^dd/dd/dddd dd:dd:dd;E;";
        String actualPrefixString = prefixString.replaceAll("d", "\\\\d");
        m_endEventPattern = Pattern.compile(actualPrefixString + ".*");
    }

    public void openLogFile() {
        try {
            FileInputStream fileInputStream = new FileInputStream(m_file);
            if (getName().matches("^.*\\.gz$")) {
                m_reader = new BufferedReader(new InputStreamReader(
                    new GZIPInputStream(fileInputStream)));
            } else {
                m_reader = new BufferedReader(new InputStreamReader(
                    fileInputStream));
            }
        } catch (Exception e) {
            Log.warn(LOG, "Exception opening file " + m_file.getPath(), e);
        }
    }
    
    /**
     * Returns true if the file name matches today's date (ie the log
     * is currently being used).
     */
    boolean isActiveLog() {
        //
        //SimpleDateFormat m_dateFormatter = new SimpleDateFormat("yyyyMMdd HH:mm:ss");
        //
        SimpleDateFormat m_dateFormatter = new SimpleDateFormat(EVENT_DATE_FORMAT);
        // get Default Timezone for this host
        m_dateFormatter.setTimeZone(TimeZone.getDefault());
        Date currentDate = Calendar.getInstance().getTime();
        String formattedDate = m_dateFormatter.format(currentDate);
        //Log.info(LOG,"Checking if the batch log file is currently active on the local host");
        //Log.info(LOG,"isActiveLog: Host TimeZone = " + TimeZone.getDefault());
        //Log.info(LOG,"isActiveLog: currentDate   = " + currentDate);
        //Log.info(LOG,"isActiveLog: formattedDate = " + formattedDate);
        //Log.info(LOG,"isActiveLog: ? = " + formattedDate.equals(getName()) );
        return formattedDate.equals(getName());
    }
    
    /**
     * Parses the log file and returns an iterator over all the 
     * event records found within the file.
     */
    public void parse() {
        //Log.info(LOG, "Processing " + getFullPath());
        String line = null;
        try {
            while ((line = m_reader.readLine()) != null) {
                if (m_endEventPattern.matcher(line).matches()) {
                    EventRecord record = createEventRecord(line);
                    m_eventRecords.add(record);
                }
            }
        } catch (IOException e) {
            Log.warn(LOG, "Exception reading file " + getName(), e);
        }
    }
    
    /**
     * Returns a new EventRecord derived from the line of data 
     * (parsed from the event log). 
     */
    EventRecord createEventRecord(String line) {
        EventRecord record = new EventRecord();
        String[] fields = m_delimiter.split(line);
        String date = fields[0];
        record.setEventDate(date.substring(6, 10) + "-" +
            date.substring(0, 2) + "-" + date.substring(3, 5));
        record.setEventTime(fields[1]);
        record.setJobName(fields[3]);
        if (m_submitHost == null || m_submitHost.equals("") || m_submitHost.equals("CE_HOSTNAME")){
	    record.setSubmitHost(strip(record.getJobName(), '.'));
        }
        else{
            record.setSubmitHost(m_submitHost);
        }
        TimeZone convertedTimezone = null;
        for (int index = 4; index < fields.length; ++index) {
            if (fields[index].startsWith("end=")) {
                Time time = new Time(strip(fields[index], '='), 
                    date, record.getEventTime(), m_timezone);
                record.setStopTimeEpoch(time.getEpoch());
                record.setStopTime(time.getIso8601());
                record.setStopTimeUtc(time.getIso8601Utc());
                convertedTimezone = time.getTimeZone();
                break;
            }
        }
        if (convertedTimezone == null) {
            LOG.debug("No end time found for event record");
            convertedTimezone = TimeZone.getTimeZone("GMT");
        }
        for (int index = 4; index < fields.length; ++index) {
            String field = fields[index];
            if (field.startsWith("user=")) {
                record.setLocalUserID(strip(field, '='));
            } else if (field.startsWith("group=")) {
                record.setLocalUserGroup(strip(field, '='));
            } else if (field.startsWith("resources_used.walltime=")) {
                TimeInterval interval = new TimeInterval(strip(field, '='));
                record.setWallDuration(interval.getIso8601());
                record.setWallDurationSeconds(interval.getSeconds());
            } else if (field.startsWith("resources_used.cput=")) {
                TimeInterval interval = new TimeInterval(strip(field, '='));
                record.setCpuDuration(interval.getIso8601());
                record.setCpuDurationSeconds(interval.getSeconds());
            } else if (field.startsWith("start=")) {
                Time time = new Time(strip(field, '='), convertedTimezone);
                record.setStartTimeEpoch(time.getEpoch());
                record.setStartTime(time.getIso8601());
                record.setStartTimeUtc(time.getIso8601Utc());
            } else if (field.startsWith("resources_used.mem=")) {
                String memory = strip(field, '=');
                record.setMemoryReal(memory.replaceAll("kb", ""));
            } else if (field.startsWith("resources_used.vmem=")) {
                String memory = strip(field, '=');
                record.setMemoryVirtual(memory.replaceAll("kb", ""));
            }
        }
        return record;
    }

    /**
     * Remove the everything from the start of the string to the 
     * specified character.
     *
     * @param string string to strip.
     * @param delimiter character to look for.
     * @return string with leading text stripped off.
     */
    private String strip(String string, char delimiter) {
        int index = string.indexOf(delimiter);
        if (index != -1) {
            return string.substring(index + 1);
        }
        return string;
    }
    
    /**
     * Closes down the log file object. This is called once all meaningful 
     * data has been read from the file.
     */
    public void close() {
        if (m_reader != null) {
            try {
                m_reader.close();
            } catch (IOException e) {
                Log.warn(LOG, "Exception closing file " + getName(), e);
            }
        }
    }
    
    /**
     * Return canonical path, with built-in exception handling.
     * If an exception is thrown only the relative path is returned.
     */
    public String getFullPath() {
        String path;
        try {
            path = getCanonicalPath();
        } catch (Exception e) {
            Log.warn(LOG, "Exception trying to get filename", e);
            path = getName();
        }
        return path;
    }
    
    /**
     * Returns an iterator of all the parsed event records.
     */
    public EventRecordIterator getRecords() {
        return new EventRecordIterator(m_eventRecords.iterator());
    }
    
}
